//    v10: Rewritten from flies.2017.v3.c by John K. Kelly and Mark T. Holder
//	v11: added full p history for replicate 0


#include <stdio.h>  //printf
#include <stdlib.h> //srand, rand
#include <time.h>   //time
#include <math.h>
#include <assert.h>
#include <limits.h>
#include <string.h>

#define SpitW		0 // set to one to emit gen 10 reproductive success
#define SelectionType	1 // 0 for trunc; 1 for mult

#define AllelesPerBulk1	1000.0 // Effective number of alleles sampled into Ancestral bulk (equivalent DNA) :: note two rounds of sampling 1/1000 = 1/2000 + 1/2000
#define AllelesPerBulk2	2000.0 // Effective number of alleles sampled into Descendant bulk (equivalent DNA)
#define NoSelectedLoci	30 // This is just a maximum for arrarys (and mean dp) // actual number specified by input file (command line)
#define SigThresholdLRT	47.0	// from data of real study
#define SigThresholdCMH	71.5	// from data of real study

#define NoTargets	238 // number of target loci for LD determination
#define LowRcoefficient0	1.1 // adjustment factor to increase LD :: fraction Dmin
#define LowRcoefficient1	0.8 // adjustment factor to increase LD :: fraction greater than Dmin

#ifdef WRAPPED
    typedef struct {
        unsigned char byte;
    } core_haplotype ; 
#else
    typedef unsigned char  core_haplotype; 
#endif
typedef core_haplotype * haplo_gen_t;
typedef haplo_gen_t * ind_gen_t;
typedef ind_gen_t * pop_gen_t;

unsigned int nSNPs 	= 291272; 	// 1987894 for allchroms.LRT.txt
unsigned int nFemales 	= 500;	// reset at command line
unsigned int nMales 	= 500;	// reset at command line
unsigned int nGen 	= 14;		// gens of SELECTION
unsigned int nReps 	= 3;
unsigned int LDYN 	= 1; 		// set to 1 to include LD

float fraction_selected = 0.63; 	// truncation selection (reset at command line)
float sc = 0.133;			// mult selection (reset at command line)

// float srange = 0.0; // range of:: s =sc+2.0*srange*(rand_uniform()-0.5);
float beta =1.0;  // selection exponent
float slocus[NoSelectedLoci];
long  List_o_targets[NoTargets];

pop_gen_t fem_pop;  //[nF][2][SNPs] 
pop_gen_t n_fem_pop; //[nF][2][SNPs]
pop_gen_t male_pop ; //[nM][2][SNPs]
pop_gen_t n_male_pop; // [nM][2][SNPs]

double * cumulativeValueM; //[nM]
double * fitnessValueM; //[nM]
double * cumulativeValueF; //[nF],
double * fitnessValueF; //[nF]
double 	qprob[100],qstat[20],Nullvar[3][5]; // [rep][chrom]	
long 	FirstSNP[5],cct[20];
float 	chromRtotal[5];
float 	MAFcdf[5];
float 	Rvals[5][100];
long  	Rpoints[5][100];

int	RealizedW[2000];  // valid only if total pop size < 2001

long  *SNPpos; //[SNPs]
int   *SNPchrom;
int   *SNPtype;
int   **SNPm;
float * QQ; //[SNPs]
float * LD;
float * Qm; //[SNPs]
float * Qf; //[SNPs]
int ** SNPrefA; //[SNPs][REPS]
int ** SNPrefD; //[SNPs][REPS]
int ** SNPaltA; //[SNPs][REPS]
int ** SNPaltD; //[SNPs][REPS]
float ** QQend; //[SNPs][REPS]
float ** QQsrt; //[SNPs][REPS]
float ** QQrp0; //[SNPs][generation]
float **** LDXc; 
float **** LDAc; 
long  *TARGETsnp;


void * alloc_or_exit(unsigned long sz) {
    void * x = malloc(sz);
    if (x == 0) {
        fprintf(stderr, "Not enough memory to allocate vars.\n");
        exit(1);        
    }
    return x;
}

core_haplotype * alloc_arr_haplotype(unsigned f) {
    return (core_haplotype *) alloc_or_exit(f * sizeof(core_haplotype));
}

int * alloc_arr_int(unsigned f) {
    return (int *) alloc_or_exit(f * sizeof(int));
}

double * alloc_arr_double(unsigned f) {
    return (double *) alloc_or_exit(f * sizeof(double));
}
float * alloc_arr_float(unsigned f) {
    return (float *) alloc_or_exit(f * sizeof(float));
}
long * alloc_arr_long(unsigned f) {
    return (long *) alloc_or_exit(f * sizeof(long));
}

ind_gen_t alloc_genotype(unsigned f, unsigned int s) {
    unsigned int i;
    ind_gen_t r = (ind_gen_t) alloc_or_exit(f* sizeof(ind_gen_t));
    for (i=0 ; i < f; ++i) {
        r[i] = alloc_arr_haplotype(s);
    }
    return r;
}

int **alloc_2d_int(unsigned f, unsigned int s) {
    unsigned int i;
    int ** r = (int **) alloc_or_exit(f* sizeof(int *));
    for (i=0 ; i < f; ++i) {
        r[i] = alloc_arr_int(s);
    }
    return r;
}

long **alloc_2d_long(unsigned f, unsigned int s) {
    unsigned int i;
    long ** r = (long **) alloc_or_exit(f* sizeof(long *));
    for (i=0 ; i < f; ++i) {
        r[i] = alloc_arr_long(s);
    }
    return r;
}

float **alloc_2d_float(unsigned f, unsigned int s) {
    unsigned int i;
    float ** r = (float **) alloc_or_exit(f* sizeof(float *));
    for (i=0 ; i < f; ++i) {
        r[i] = alloc_arr_float(s);
    }
    return r;
}

float ***alloc_3d_float(unsigned f, unsigned int s, unsigned int t) {
    unsigned int i;
    float *** r = (float ***) alloc_or_exit(f* sizeof(float *));
    for (i=0 ; i < f; ++i) {
        r[i] = alloc_2d_float(s,t);
    }
    return r;
}

float ****alloc_4d_float(unsigned f, unsigned int s, unsigned int t, unsigned int u) {
    unsigned int i;
    float **** r = (float ****) alloc_or_exit(f* sizeof(float *));
    for (i=0 ; i < f; ++i) {
        r[i] = alloc_3d_float(s,t,u);
    }
    return r;
}

int ***alloc_3d_int(unsigned int f, unsigned int s, unsigned int t) {
    unsigned int i;
    int *** r = (int ***) alloc_or_exit(f* sizeof(int *));
    for (i=0 ; i < f; ++i) {
        r[i] = alloc_2d_int(s, t);
    }
    return r;
}

pop_gen_t alloc_pop(unsigned int f, unsigned int s, unsigned int t) {
    unsigned int i;
    pop_gen_t r = (pop_gen_t) alloc_or_exit(f* sizeof(pop_gen_t));
    for (i=0 ; i < f; ++i) {
        r[i] = alloc_genotype(s, t);
    }
    return r;
}

void alloc_globals(unsigned int n_snps, unsigned int n_f, unsigned int n_m, unsigned int n_r) {

	unsigned int n_bits_for_hap = 1 + (n_snps/8);
	fem_pop = alloc_pop(n_f, 2, n_bits_for_hap);
	n_fem_pop = alloc_pop(n_f, 2, n_bits_for_hap);
	cumulativeValueF  = alloc_arr_double(n_f);
	fitnessValueF  = alloc_arr_double(n_f);

	male_pop = alloc_pop(n_m, 2, n_bits_for_hap);
	n_male_pop = alloc_pop(n_m, 2, n_bits_for_hap);
	cumulativeValueM  = alloc_arr_double(n_m);
	fitnessValueM  = alloc_arr_double(n_m);
	Qm = alloc_arr_float(n_snps);
	Qf = alloc_arr_float(n_snps);
	QQ = alloc_arr_float(n_snps);
	TARGETsnp = alloc_arr_long(n_snps);
	LD = alloc_arr_float(n_snps);
	SNPpos = alloc_arr_long(n_snps); //[SNPs]
	SNPchrom = alloc_arr_int(n_snps);
	SNPtype = alloc_arr_int(n_snps);
	SNPm  = alloc_2d_int(n_snps, 6); // read depths, 6 pops
	SNPrefD = alloc_2d_int(n_snps, n_r);
	SNPrefA = alloc_2d_int(n_snps, n_r);
	SNPaltA = alloc_2d_int(n_snps, n_r);
	SNPaltD = alloc_2d_int(n_snps, n_r);
	QQend = alloc_2d_float(n_snps, n_r);
	QQsrt = alloc_2d_float(n_snps, n_r);
	QQrp0 = alloc_2d_float(n_snps, 20); // valid up to 19 gens of selection

	LDAc = alloc_4d_float(6,26,26,11); //[q1][q2][dist][int]
	LDXc = alloc_4d_float(6,26,26,11);
    
}


char* concat(const char *s1, const char *s2)
{
    char *result = malloc(strlen(s1)+strlen(s2)+1);//+1 for the zero-terminator
    strcpy(result, s1);
    strcat(result, s2);
    return result;
}


unsigned int parseIntOrExit(const char *arg, const char *name) {
    unsigned long tmp;
    char * convptr;
    tmp = strtoul(arg, &convptr, 10);
    if (tmp == 0 || *convptr != 0) {
        fprintf(stderr, "Could not interpret argument \"%s\" as %s.\n", arg, name);
        exit(1);
    }
    if (tmp >= UINT_MAX) {
        fprintf(stderr, "Could not store argument \"%s\" as an int.\n", arg);
        exit(1);
    }
    return (unsigned int) tmp;
}

void doSimRep(int rp, FILE * fd, FILE * fe);
void selectedSNPs();
void FitnessVar(int rp, FILE * fd, pop_gen_t male_pop, pop_gen_t fem_pop);
void makeLD(FILE * fLD);
double rand_uniform();
double  generateGaussianNoise(double mu, double sigma, double x1, double x2);
void   FITNESSM(),FITNESSF();
void    Founding_Pop();
unsigned int     selectFitM(),selectFitF(); /* selecting an individual based on fitness + phenotype */
int     POISSON(double m);
void    Analyze_Seqs(int gen, FILE *fzim, FILE *fsim, FILE *fdr, FILE *fe);

int main(int argc, char * argv[])
{
int k,vv,rp,curchrom,x,y,z,ix,iy,gen,lctyp,ss1=0,ss2=0;
long j,uxt,v1,v2,v3,lastpos=0;
double rx,pmean,LDmean,LL0,LL1,LL3,x1,x3,x2,ry,p25,p75,CMHstat,CMHmax=0.0,abschange=0.0;            // core variances dz
char inputsLRT[10][200];
char * simrepname, * spdist, *spdist2;
double inpx[12],LRTstats[15],SNPvz[3],SNPdz[3],autonv[2];
long   inpy[10],LRTdist[2][100][21],LRTbyType[11][4],CMHdist[501];
int    LDc[11];
float  LDintv[11],dzlist[5][nSNPs],QUQ[NoSelectedLoci];

    time_t now;
    unsigned int rnseed = 0;



if (argc == 8) {
	simrepname=argv[1];
	nFemales = parseIntOrExit(argv[2], "nFemales");
	nMales = parseIntOrExit(argv[3], "nMales");
	spdist=argv[4];//"s20.uniform.key.txt"
	spdist2=argv[5];//"888.qx.txt"
	if (SelectionType==0) fraction_selected=atof(argv[6]);
	if (SelectionType==1) sc=atof(argv[6]);
	rnseed = parseIntOrExit(argv[argc - 1], "rnseed"); 
} else {
    fprintf(stderr, "Args recieved %d\n",argc);
    fprintf(stderr, "Did not get right arguments:\n jf NUM_SNPS NUM_FEMALES NUM_MALES NUM_GENERATIONS NUM_REPS RN_SEED\nUsing default values");
    /* get current time from the system clock & set seed for a new sequence of pseudo-random numbers*/
    if (rnseed == 0) {
        time(&now);
        rnseed = (unsigned int) now; 
    }
}
fprintf(stderr, "%s\n",simrepname);
fprintf(stderr, "nSNPs = %u\n", nSNPs);
fprintf(stderr, "nFemales = %u\n", nFemales);
fprintf(stderr, "nMales = %u\n", nMales);
fprintf(stderr, "nGen = %u\n", nGen);
fprintf(stderr, "nReps = %u\n", nReps);
fprintf(stderr, "seed = %u\n", rnseed);
alloc_globals(nSNPs, nFemales, nMales, nReps);

char* sname1 = concat(simrepname, ".selected_loci.txt");
char* sname2 = concat(simrepname, ".LRT.stats.txt");
char* sname3 = concat(simrepname, ".CMH.dist.txt");
char* sname4 = concat(simrepname, ".null.var.txt");
char* sname5 = concat(simrepname, ".simple.txt");
char* sname6 = concat(simrepname, ".LRT.sig.txt");
char* sname7 = concat(simrepname, ".CMH.sig.txt");
char* sname8 = concat(simrepname, ".reproductive.success.txt");
char* sname9 = concat(simrepname, ".dpcovar.txt");

const char * file1 = sname1;
const char * file2 = sname2;
const char * file11 =sname3;
const char * file12 =sname4;
const char * file13 =sname5;
const char * file14 =sname6;
const char * file15 =sname7;
const char * file16 =sname8;
const char * file17 =sname9;

const char * file22 ="Auto.hash.txt";
const char * file20 ="X.hash.txt";
const char * file23 ="Rmap.Dsim.v2.txt";
const char * file9 = "pos.q.v2.txt";
const char * file8 = "Sim.LD.txt";
const char * file0 = spdist; // key to sites:: e.g. s20.uniform.key.txt
const char * file3 = spdist2; // key to sites:: e.g. s20.uniform.key.txt
int fsfret;


FILE *fd, *fcdf,*fcdq, *fmp,*fde,*fky,*fkz,*fdf,*fLD, *fe, *flb, *flc, *flz, *fdg, *fdh, *fdi;
fd = fopen(file1,"w+");
fe = fopen(file2,"w+");
fde = fopen(file11,"w+");
fkz = fopen(file16,"w+");
fky = fopen(file17,"w+");
fdf = fopen(file12,"w+");
fLD = fopen(file8,"w+");
fcdf= fopen(file0,"r");
fcdq= fopen(file3,"r");
fmp = fopen(file9,"r");
flc = fopen(file20,"r");

flb = fopen(file22,"r");
flz = fopen(file23,"r");
fdg = fopen(file13,"w+");
fdh = fopen(file14,"w+");
fdi = fopen(file15,"w+");

  srand(rnseed);


/*R probs per chrom arm :: D sim*/
/* Rvals and Rpoints are locations specific information about recombination probabilities :: D sim */
chromRtotal[0]=0.57;
chromRtotal[1]=0.655;
chromRtotal[2]=0.555;
chromRtotal[3]=0.83;
chromRtotal[4]=0.59;	
/* cdf for MAF of selected sites */
MAFcdf[0]=0.0145;
MAFcdf[1]=0.1232;
MAFcdf[2]=0.4638;
MAFcdf[3]=0.7536;
MAFcdf[4]=1.00;

x=0;
vv=0;
while(fscanf(flz,"%d\t%lf\t%ld\t%ld\t%ld\n",&k,&x1,&v1,&v2,&v3) != EOF)
{
// 4	0	876596	872350  5599
	if (k != x) { 	vv=0;
			x=k; }
	Rvals[k][vv]  =x1;
	Rpoints[k][vv]=v3; // position on snp in big list (0-292k)
//	printf("%d  %d %f %ld\n",k,vv,Rvals[k][vv],Rpoints[k][vv]);
	vv++;
}


while(fscanf(fcdq,"%d\t%lf\n",&k,&rx) != EOF) QUQ[k]=rx; // NoSelectedLoci


ix=0;
iy=0;
pmean=0.0;
for(j=0;j<nSNPs;j++) {

//0	181	0.99455040872	134	113	120	105	174	638
    fsfret = fscanf(fmp,"%ld\t%ld\t%lf\t%ld\t%ld\t%ld\t%ld\t%ld\t%ld\n",&uxt,&inpy[0],&x1,&inpy[1],&inpy[2],&inpy[3],&inpy[4],&inpy[5],&inpy[6]);   
    assert(fsfret != EOF && fsfret > 0);

// 0	0	177	2	-99
// 1	0	181	0	0
    fscanf(fcdf,"%ld\t%d\t%ld\t%d\t%ld\n",&v1,&k,&inpy[8],&lctyp,&v2);   
	if(j!=v1) printf("accounting error 1\n");
	if(inpy[0] != inpy[8]) printf("accounting error 2\n");
	SNPchrom[j]=k;
	if(inpy[0]==lastpos) inpy[0]++;
	SNPpos[j]=inpy[0]; //[SNPs]
	lastpos=inpy[0];
	for(vv=0;vv<6;vv++) SNPm[j][vv]= (int)inpy[vv+1]; // read depths

	TARGETsnp[j]=v2; // -1: Auto-regular, -2: X-regular, -3: Low r LD



	if(lctyp<1) {
		QQ[j]=(float)x1;
		if (QQ[j]<0.005) QQ[j]=0.005;
		if (QQ[j]>0.995) QQ[j]=0.995;
		SNPtype[j]=0; // neutral
	}
	else {
		x1=rand_uniform();
		QQ[j]=(float)x1;
		if (QQ[j]<0.005) QQ[j]=0.005;
		if (QQ[j]>0.995) QQ[j]=0.995;
/*
		if(x1<MAFcdf[0]) 	QQ[j]=0.01+0.09*rand_uniform();
		else if(x1<MAFcdf[1]) 	QQ[j]=0.1+0.1*rand_uniform();
		else if(x1<MAFcdf[2]) 	QQ[j]=0.2+0.1*rand_uniform();
		else if(x1<MAFcdf[3]) 	QQ[j]=0.3+0.1*rand_uniform();
		else			QQ[j]=0.4+0.1*rand_uniform();

		if (rand_uniform()<0.5) QQ[j]=1.0-QQ[j];*/

		List_o_targets[iy]=j;
		iy++;
		if (lctyp==2) {
			if (rand_uniform()<0.5) SNPtype[j]=1; // positive effect
			else SNPtype[j]=-1; // negative effect
			if (SelectionType==0) slocus[ix]=1.0;
			if (SelectionType==1) slocus[ix]=sc;
			QQ[j]=QUQ[ix];
			if (QQ[j]<0.2) SNPtype[j]=1;
			if (QQ[j]>0.8) SNPtype[j]=-1;
			ix++;
		}
	}
	pmean+=QQ[j];
	
    if(j==0) {
        FirstSNP[0]=0; //2L
        curchrom=uxt;
    }
    if (uxt != curchrom) {
        FirstSNP[k]=j;
        curchrom=uxt;
	printf("Reading %d start snp pos %ld \n", curchrom,j );
    }
}
printf( "mean p %lf\n",pmean/(float)nSNPs );

if (LDYN==1) {


	while(fscanf(flb,"%d\t%d\t%d\t%d\t%ld\t%lf\n",&LDc[0],&LDc[1],&LDc[2],&LDc[3],&v1,&x1) != EOF) // Auto LD hash
		LDAc[LDc[0]][LDc[1]][LDc[2]][LDc[3]]=(float)x1;

	while(fscanf(flc,"%d\t%d\t%d\t%d\t%ld\t%lf\n",&LDc[0],&LDc[1],&LDc[2],&LDc[3],&v1,&x1) != EOF) // X LD hash
		LDXc[LDc[0]][LDc[1]][LDc[2]][LDc[3]]=(float)x1;

	printf("Read LD tables\n");

	for(x=0;x<6;x++) // make probs to cum probs
		for(y=0;y<=25;y++)
			for(z=0;z<=y;z++)
				for(k=1;k<11;k++) {
					LDAc[x][y][z][k]+=LDAc[x][y][z][k-1]; 
					if(LDAc[x][y][z][k]>1.0) LDAc[x][y][z][k]=1.0;
					LDXc[x][y][z][k]+=LDXc[x][y][z][k-1]; 
					if(LDXc[x][y][z][k]>1.0) LDXc[x][y][z][k]=1.0;
				}
} // end if



if (LDYN==1) makeLD(fLD);  // LD in ancestral pop
else
	for(j=0;j<nSNPs;j++) LD[j]=0.0;	

printf("Finished making LD \n");

for(rp=0;rp<nReps;rp++) {
    doSimRep(rp, fd, fe);
	for(j=0;j<nMales;j++) fprintf(fkz,"%d\t%ld\t%d\n",rp,j,RealizedW[j]);
	for(j=0;j<nFemales;j++) fprintf(fkz,"%d\t%ld\t%d\n",rp,j,RealizedW[j+1000]);
}



for(j=0;j<15;j++) LRTstats[j]=0.0;
for(j=0;j<2;j++) for(x=0;x<100;x++) for(k=0;k<21;k++) LRTdist[j][x][k]=0;
for(j=0;j<4;j++)  for(k=0;k<11;k++) LRTbyType[k][j]=0;
for(j=0;j<501;j++) CMHdist[j]=0;
ry=0.0;
for(j=0;j<nSNPs;j++)   // Currently valid only for REPs = 3
{
	x1 = 0.0; // numer of CMHstat
	x2 = 0.0; // denom of CMHstat
	for(rp=0;rp<nReps;rp++) {
		x3=(float)(SNPrefA[j][rp]+SNPaltA[j][rp]+SNPrefD[j][rp]+SNPaltD[j][rp]);
		x1+=( (float)SNPrefA[j][rp] - (float)(SNPrefA[j][rp]+SNPaltA[j][rp])*(float)(SNPrefA[j][rp]+SNPrefD[j][rp])/x3 );
		x2+=( (float)(SNPrefA[j][rp]+SNPaltA[j][rp])*(SNPrefD[j][rp]+SNPaltD[j][rp])*(SNPrefA[j][rp]+SNPrefD[j][rp])*(SNPaltA[j][rp]+SNPaltD[j][rp]) /(x3*x3*(x3-1.0)) );
	}
	if(x2>0.0) CMHstat=x1*x1/x2;
	else CMHstat=0.0;//printf("%ld CHMstats %lf %lf %lf\n",j,x1,x2,CMHstat);
	if(CMHstat>CMHmax) CMHmax=CMHstat;
	if( CMHstat > SigThresholdCMH) 
	{	fprintf(fdi,"%d\t%ld\t%f\t%f\t%f\t%lf\t",SNPchrom[j],SNPpos[j],QQ[j],(QQsrt[j][0]+QQsrt[j][1]+QQsrt[j][2])/3.0,(QQend[j][0]+QQend[j][1]+QQend[j][2])/3.0,CMHstat );
		v1=250000;
		if(SNPtype[j]==1 || SNPtype[j]==-1) v1 =0;
		else if(SNPtype[TARGETsnp[j]]==1 || SNPtype[TARGETsnp[j]]==-1) v1=SNPpos[j]-SNPpos[TARGETsnp[j]];
		fprintf(fdi,"%ld\t%ld\n",v1,TARGETsnp[TARGETsnp[j]]); 
		ss1++;}

	ry+=CMHstat;
	k = (int)CMHstat;
	if(k>500) k =500;
	CMHdist[k]++;
}
printf("Mean CHMstat %lf\n",ry/(float)nSNPs);


for(j=0;j<501;j++) fprintf(fde,"%ld\t%ld\n",j,CMHdist[j]); 

autonv[0]=0.0;
autonv[1]=0.0;

for(rp=0;rp<nReps;rp++)
{
	for(k = 0; k < 20; k++) {
	qstat[k] = 0.0;
	cct[k] = 0;
	}
	for(j=0;j<nSNPs;j++) 
	{
/// All this within rp below
	x1=(float)SNPrefA[j][rp]/(float)(SNPrefA[j][rp]+SNPaltA[j][rp]);
	x2=(float)SNPrefD[j][rp]/(float)(SNPrefD[j][rp]+SNPaltD[j][rp]);
		if(x1>=0.05 && x1<=0.95) // use for null var
		{
		ry = 2*asin(sqrt(x2)) - 2*asin(sqrt(x1)) ;   // change in transformed allele freq
		    if (j<FirstSNP[1]) {     // 2L
		        qstat[0]+= ry ;
		        qstat[1]+= ry*ry;
			if (cct[0] % 2 == 0) 	dzlist[0][cct[0]]=ry;
			else 			dzlist[0][cct[0]]=-ry;
		        cct[0]++;
		        qstat[10]+= (1.0/(float)(SNPrefA[j][rp]+SNPaltA[j][rp])) + (1.0/(float)(SNPrefD[j][rp]+SNPaltD[j][rp]));
		    } else if (j<FirstSNP[2]) { // 2R         
		        qstat[2]+= ry ;
		        qstat[3]+= ry*ry;
			if (cct[1] % 2 == 0) 	dzlist[1][cct[1]]=ry;
			else 			dzlist[1][cct[1]]=-ry;
		        cct[1]++;
		        qstat[11]+= (1.0/(float)(SNPrefA[j][rp]+SNPaltA[j][rp])) + (1.0/(float)(SNPrefD[j][rp]+SNPaltD[j][rp]));
		    } else if (j<FirstSNP[3]) { // 3L        
		        qstat[4]+= ry ;
		        qstat[5]+= ry*ry;
			if (cct[2] % 2 == 0) 	dzlist[2][cct[2]]=ry;
			else 			dzlist[2][cct[2]]=-ry;
		        cct[2]++;
		        qstat[12]+= (1.0/(float)(SNPrefA[j][rp]+SNPaltA[j][rp])) + (1.0/(float)(SNPrefD[j][rp]+SNPaltD[j][rp]));
		    } else if (j<FirstSNP[4]) { // 3R        
		        qstat[6]+= ry ;
		        qstat[7]+= ry*ry;
			if (cct[3] % 2 == 0) 	dzlist[3][cct[3]]=ry;
			else 			dzlist[3][cct[3]]=-ry;
		        cct[3]++;
		        qstat[13]+= (1.0/(float)(SNPrefA[j][rp]+SNPaltA[j][rp])) + (1.0/(float)(SNPrefD[j][rp]+SNPaltD[j][rp]));
		    } else  { // X       
		        qstat[8]+= ry ;
		        qstat[9]+= ry*ry;
			if (cct[4] % 2 == 0) 	dzlist[4][cct[4]]=ry;
			else 			dzlist[4][cct[4]]=-ry;
		        cct[4]++;
		        qstat[14]+= (1.0/(float)(SNPrefA[j][rp]+SNPaltA[j][rp])) + (1.0/(float)(SNPrefD[j][rp]+SNPaltD[j][rp]));
		    }
		}

	} // end j loop

	for(j=0;j<5;j++) {

		// find 25th percentile and 75th percentile
		p25=-9;
		p75=-9;
		for(v1=0;v1<cct[j];v1++) {
			uxt=0; 
			for(v2=0;v2<cct[j];v2++) if(dzlist[j][v1]>dzlist[j][v2]) uxt=uxt+1;
		        x1= (float)uxt/(float)(cct[j]-1);
			if (x1>0.2475 && x1<0.2526) p25=dzlist[j][v1];
			if (x1>0.7475 && x1<0.7526) p75=dzlist[j][v1];
			if (p25 != -9 && p75 != -9) break;
		}
		qstat[2*j]=qstat[2*j]/(float)cct[j]; // mean dz
		qstat[10+j]=qstat[10+j]/(float)cct[j]; // read depth var 
		Nullvar[rp][j]=((p75-p25)/1.349)*((p75-p25)/1.349) - qstat[10+j]; // robust estimator - (read depth var)
		if(j<4) {	autonv[0]+=(Nullvar[rp][j]/12.0);
				autonv[1]+=(Nullvar[rp][j]*Nullvar[rp][j]/12.0);}
		printf("chrom,Null mean, p25,p75, RDvar,robust null var \t%ld\t%lf\t%lf\t%lf\t%lf\t%lf\n",j,qstat[2*j],p25,p75,qstat[10+j],Nullvar[rp][j]);
		fprintf(fdf,"%ld\t%lf\n",j,Nullvar[rp][j]);
	} // end j (chrom) loop


} // end rp loop
autonv[1]=1.091*(autonv[1]-(autonv[0]*autonv[0]));//sample size corrected variance

for(j=0;j<nSNPs;j++)
{
	LL0=0.0; //  null model
	LL1=0.0; // parallel change model
	LL3=0.0; // free change model
	if (j<FirstSNP[1]) y=0;     // 2L
	else if (j<FirstSNP[2])y=1; // 2R
	else if (j<FirstSNP[3])y=2; // 
	else if (j<FirstSNP[4])y=3; 
	else y=4;

	for (rp=0;rp<3;rp++) {   
		x1=(float)SNPrefA[j][rp]/(float)(SNPrefA[j][rp]+SNPaltA[j][rp]);
		x2=(float)SNPrefD[j][rp]/(float)(SNPrefD[j][rp]+SNPaltD[j][rp]);
		SNPdz[rp] = 2*asin(sqrt(x2)) - 2*asin(sqrt(x1)) ;   // change in transformed allele freq
		SNPvz[rp] = Nullvar[rp][y] + (1.0/(float)(SNPrefA[j][rp]+SNPaltA[j][rp])) + (1.0/(float)(SNPrefD[j][rp]+SNPaltD[j][rp])); // variance in change
		LL0-= ( (SNPdz[rp]*SNPdz[rp])/(2.0*SNPvz[rp]) + log(SNPvz[rp])/2.0 ); }
    
    rx = ( pow(SNPvz[0],-1.0)*SNPdz[0]+pow(SNPvz[1],-1.0)*SNPdz[1]+pow(SNPvz[2],-1.0)*SNPdz[2] )/( pow(SNPvz[0],-1.0)+pow(SNPvz[1],-1.0)+pow(SNPvz[2],-1.0) ); // MLE dz
	

    for (rp=0;rp<3;rp++) {   
	LL1-= ( (SNPdz[rp]-rx)*(SNPdz[rp]-rx) /(2.0*SNPvz[rp]) + log(SNPvz[rp])/2.0 );
	LL3-= ( log(SNPvz[rp])/2.0 ); }

	
	LRTstats[0]+=(2.0*(LL1-LL0));
	LRTstats[3]+=(2.0*(LL1-LL0)*2.0*(LL1-LL0));
	LRTstats[4]+=(2.0*(LL3-LL1));
	LRTstats[5]+=(2.0*(LL3-LL1)*2.0*(LL3-LL1));

	k=(int)(2.0*(LL1-LL0));
	if(k>20) k=20;
	x=(int)(100*QQ[j]);
	LRTdist[0][x][k]++;
	k=(int)(2.0*(LL3-LL1));
	if(k>20) k=20;
	x=(int)(100*QQ[j]);
	LRTdist[1][x][k]++;

	k=(int)(2.0*(LL1-LL0)/10.0);
	if(k>10) k=10;
	if(j<FirstSNP[4]){
		LRTbyType[k][0]++;
		if (SNPtype[j]==1 || SNPtype[j]==-1) LRTbyType[k][1]++;
	} else {
		LRTbyType[k][2]++;
		if (SNPtype[j]==1 || SNPtype[j]==-1) LRTbyType[k][3]++;
	}


    if( (2.0*(LL1-LL0))>LRTstats[8] ) LRTstats[8]=2.0*(LL1-LL0);
    if( (2.0*(LL3-LL1))>LRTstats[9] ) LRTstats[9]=2.0*(LL3-LL1);
    if (SNPtype[j]==1 || SNPtype[j]==-1) {
	LRTstats[1]+=1.0;
	LRTstats[2]+=(2.0*(LL1-LL0));
    }		  


//    fprintf(fkz,"%ld\t%d\t%ld\t%f\t%f\t%f\t",j,SNPchrom[j],SNPpos[j],SNPdz[0]/sqrt(SNPvz[0]),SNPdz[1]/sqrt(SNPvz[1]),SNPdz[2]/sqrt(SNPvz[2]) );
//    fprintf(fkz,"%lf\t%lf\n",2.0*(LL1-LL0),2.0*(LL3-LL1) ); 

	if( 2.0*(LL1-LL0) > SigThresholdLRT) {
		fprintf(fdh,"%d\t%ld\t%f\t%f\t%f\t%f\t%f\t%lf\t",SNPchrom[j],SNPpos[j],QQ[j],(QQsrt[j][0]+QQsrt[j][1]+QQsrt[j][2])/3.0,SNPdz[0]/sqrt(SNPvz[0]),SNPdz[1]/sqrt(SNPvz[1]),SNPdz[2]/sqrt(SNPvz[2]),2.0*(LL1-LL0) );
		v1=250000;
		if(SNPtype[j]==1 || SNPtype[j]==-1) v1 =0;
		else if(SNPtype[TARGETsnp[j]]==1 || SNPtype[TARGETsnp[j]]==-1) v1=SNPpos[j]-SNPpos[TARGETsnp[j]];
		fprintf(fdh,"%ld\t%ld\n",v1,TARGETsnp[TARGETsnp[j]]); 
		ss2++;}

	if(SNPtype[j]==1 || SNPtype[j]==-1) { 
		fprintf(fd,"%d\t%ld\t%f\t%f\t%lf\n",SNPchrom[j],SNPpos[j],(QQsrt[j][0]+QQsrt[j][1]+QQsrt[j][2])/3.0,(QQend[j][0]+QQend[j][1]+QQend[j][2])/3.0,2.0*(LL1-LL0) );
		rx = (QQend[j][0]+QQend[j][1]+QQend[j][2])/3.0 - (QQsrt[j][0]+QQsrt[j][1]+QQsrt[j][2])/3.0;
		if(rx<0.0) rx=(-rx);
		abschange+=rx/(float)NoSelectedLoci; }


} // end j (snps)

// analyze (co)variance of dp in rep0
for(gen=0;gen<nGen;gen++) {
	LRTstats[10]=0.0;
	LRTstats[11]=0.0;
	for(j=0;j<nSNPs;j++) {
		if (QQrp0[j][gen]<0.0) QQrp0[j][gen]=0;
		if (QQrp0[j][gen]>1.0) QQrp0[j][gen]=1.0;
		QQrp0[j][gen]=QQrp0[j][gen+1]-QQrp0[j][gen]; // variable is now dp
		LRTstats[10]+=(double)QQrp0[j][gen];
		LRTstats[11]+=(double)(QQrp0[j][gen]*QQrp0[j][gen]);
		}
	LRTstats[10]=LRTstats[10]/(double)nSNPs; //mean change
	LRTstats[11]=LRTstats[11]/(double)nSNPs - (LRTstats[10]*LRTstats[10]); //var dp
	printf("gen %d Mean dp %lf var dp %lf\n",gen,LRTstats[10],LRTstats[11]);
	fprintf(fky,"%6d\t%12.10lf\t%12.10lf\n",gen,LRTstats[10],LRTstats[11]);
}
	LRTstats[10]=0.0; // variance in total change
	LRTstats[11]=0.0;
	for(j=0;j<nSNPs;j++) {
		LRTstats[10]+=(double)QQrp0[j][19]; // total DP
		LRTstats[11]+=(double)(QQrp0[j][19]*QQrp0[j][19]);
		}
	LRTstats[10]=LRTstats[10]/(double)nSNPs; //mean change
	LRTstats[11]=LRTstats[11]/(double)nSNPs - (LRTstats[10]*LRTstats[10]); //var dp
	printf("Total Mean dp %lf var dp %lf\n",LRTstats[10],LRTstats[11]);
	fprintf(fky,"all\t%12.10lf\t%12.10lf\n",LRTstats[10],LRTstats[11]);

for(gen=0;gen<(nGen-1);gen++) {
	LRTstats[10]=0.0;
	LRTstats[11]=0.0;
	LRTstats[12]=0.0;
	for(j=0;j<nSNPs;j++) {
		LRTstats[10]+=QQrp0[j][gen];
		LRTstats[11]+=QQrp0[j][gen+1];
		LRTstats[12]+=(QQrp0[j][gen+1]*QQrp0[j][gen]);
		}
	LRTstats[10]=LRTstats[10]/(double)nSNPs; //mean change gen
	LRTstats[11]=LRTstats[11]/(double)nSNPs; //mean change gen+1
	LRTstats[12]=LRTstats[12]/(double)nSNPs - (LRTstats[10]*LRTstats[11]); //covar dp
	printf("gen %d cov dp %lf\n",gen,LRTstats[12]);
	fprintf(fky,"%6d\tcov\t%12.10lf\n",gen,LRTstats[12]);
}

for(k=0;k<11;k++) fprintf(fe,"LRT cat\t%d\t%ld\t%ld\t%ld\t%ld\n",k,LRTbyType[k][0],LRTbyType[k][1],LRTbyType[k][2],LRTbyType[k][3]);

LRTstats[0]=LRTstats[0]/(float)nSNPs;
LRTstats[3]=LRTstats[3]/(float)nSNPs -(LRTstats[0]*LRTstats[0]); // variance LRT-1
LRTstats[4]=LRTstats[4]/(float)nSNPs;
LRTstats[5]=LRTstats[5]/(float)nSNPs -(LRTstats[4]*LRTstats[4]); // variance LRT-2

LRTstats[2]=LRTstats[2]/(float)LRTstats[1];
printf("LRT mean all %lf\n",LRTstats[0]);
printf("LRT mean selected %lf\n",LRTstats[2]);

fprintf(fdg,"CMHmax %lf CMHsig %d LRTsig %d v auto %lf %lf mean DP %lf LRT-1 MV %lf %lf LRT-2 MV %lf %lf\n",CMHmax,ss1,ss2,autonv[0],autonv[1],abschange,LRTstats[0],LRTstats[3],LRTstats[4],LRTstats[5]);

return 0;
}    /* End of main program */




double rand_uniform() {
    return (double) rand() / (1.0 + (double)RAND_MAX);
}

void simulateGen(long gen);
unsigned char bit_as_char(const core_haplotype * hg, unsigned int index);
void copy_bit(const core_haplotype * src, unsigned int index, core_haplotype * dest);
float minFloat ( float a, float b );
void set_bit(core_haplotype * dest, unsigned int index, unsigned int allele);

unsigned char BIT_INDEX_TO_BIT[] = {1, 2, 4, 8, 16, 32, 64, 128 }; 
unsigned char INV_BIT_INDEX_TO_BIT[] = {0xFE, 0xFD, 0xFB, 0xF7, 0xEF, 0xDF, 0xBF, 0x7F }; 
unsigned char bit_as_char(const core_haplotype * hg, unsigned int index) {
    const unsigned int compressed_index = index / 8;
    const unsigned int bit_index = index % 8;
    const unsigned char bit = BIT_INDEX_TO_BIT[bit_index];
    return (hg[compressed_index] & bit ? 1 : 0);
}

float minFloat ( float a, float b ) { return a < b ? a : b; }


void copy_bit(const core_haplotype * src, unsigned int index, core_haplotype * dest) {
    const unsigned int compressed_index = index / 8;
    const unsigned int bit_index = index % 8;
    const unsigned char bit = BIT_INDEX_TO_BIT[bit_index];
    if (src[compressed_index] & bit) {
        dest[compressed_index] |= bit;
    } else {
        dest[compressed_index] &= INV_BIT_INDEX_TO_BIT[bit_index];
    }
}

void set_bit(core_haplotype * dest, unsigned int index, unsigned int allele) {
    const unsigned int compressed_index = index / 8;
    const unsigned int bit_index = index % 8;
    if (allele==1) {
        dest[compressed_index] |= BIT_INDEX_TO_BIT[bit_index];
    } else {
        dest[compressed_index] &= INV_BIT_INDEX_TO_BIT[bit_index];
    }
}

void sum_diploid(unsigned int kstart, unsigned int kend, unsigned int n, float * q_arr, const pop_gen_t pop) {
    int k, i;
    for (k = kstart; k < kend; ++k) {
        q_arr[k] = 0.0;
        for (i = 0; i < n; ++i) {
            q_arr[k]+=(float)(bit_as_char(pop[i][0], k) + bit_as_char(pop[i][1], k))/(float)(2*n);
        }
    }
}
void sum_haploid(unsigned int kstart, unsigned int kend, unsigned int n, float * q_arr, const pop_gen_t pop) {
    int k, i;
    for (k = kstart; k < kend; ++k) {
        q_arr[k] = 0.0;
        for (i = 0; i < n; ++i) {
            q_arr[k] += (float)(bit_as_char(pop[i][1], k))/(float)(n);
        }
    }
}

void makeLD( FILE * fLD) {
	float qr,qx,dmin,dmax;
	long k,dd,k2;
	int x,i,dcat,catA,catB,catr,catx;
	double x1,x2,meanr2[10][2];

	for(k=0;k<10;k++)
		for(i=0;i<2;i++) meanr2[k][i] = 0.0;

/* no LD among focal sites */
	for(x=0;x<NoTargets;x++) LD[List_o_targets[x]]=0.0;


/* rest of sites */ 
   for(k=0;k<nSNPs;k++) {

	//printf("uu 77 %ld %ld\n",k,TARGETsnp[k]);

        if(TARGETsnp[k] >= 0) // not a focal snp
        {
		dd=abs(SNPpos[k]-SNPpos[ TARGETsnp[k] ]);
		if (dd < 10)         dcat=0;
		else if (dd < 100)   dcat=1;
		else if (dd < 1000)  dcat=2;
		else if (dd < 10000) dcat=3;
		else if (dd < 100000) dcat=4;
		else dcat=5;
		
		x1 = rand_uniform();
            qr = minFloat( QQ[ TARGETsnp[k] ], 1.0 - QQ[ TARGETsnp[k] ] );
            qx = minFloat( QQ[ k ], 1.0 - QQ[ k ] );
            catr=(int)(50.0*qr);
            catx=(int)(50.0*qx);
            if(catr<catx) { catA= catx;
                            catB= catr;}
            else          { catA= catr;
                            catB= catx;}
            dmin= -qr*qx;
		if(qr<qx) dmax=qr*(1.0-qx);
		else 	dmax=qx*(1.0-qr);


// -1: Auto-regular, -2: X-regular, -3: Low r LD
		if (TARGETsnp[ TARGETsnp[k] ] == -1){
	                for(i=0;i<=10;i++) {
			    if (LDAc[dcat][catA][catB][i] >= x1) {
			    LD[k] = dmin + (float)i*(dmax-dmin)*0.1;
			    break; }						
			}
		} // end hash Auto	
		else if (TARGETsnp[ TARGETsnp[k] ] == -2){
	                for(i=0;i<=10;i++) {
			    if (LDXc[dcat][catA][catB][i] >= x1) {
			    LD[k] = dmin + (float)i*(dmax-dmin)*0.1;
			    break; }						
			}
		} // end hash X
		else if (TARGETsnp[ TARGETsnp[k] ] == -3)
		{
			if(k<FirstSNP[4]) // autosomal
			{	LD[k] = dmax;
				for(i=0;i<10;i++) {
					if(i==0) x2=LowRcoefficient0*LDAc[dcat][catA][catB][i];
					else  x2=LowRcoefficient1*LDAc[dcat][catA][catB][i];
					if (x2 >= x1) {
					LD[k] = dmin + (float)i*(dmax-dmin)*0.1;
					break; }
				}
			} else
			{	LD[k] = dmax;
				for(i=0;i<10;i++) {
					if(i==0) x2=LowRcoefficient0*LDXc[dcat][catA][catB][i];
					else  x2=LowRcoefficient1*LDXc[dcat][catA][catB][i];
					if (x2 >= x1) {
					LD[k] = dmin + (float)i*(dmax-dmin)*0.1;
					break; }
				}
			}

		} // end hash Low R

		else printf("Error 77 %ld %ld\n",TARGETsnp[k],TARGETsnp[ TARGETsnp[k] ]);
		x1=  LD[k]*LD[k]/( qr*qx*(1-qr)*(1-qx) ) ;
//		fprintf(fLD,"%ld\t%ld\t%f\t%f\t%f\t%f\t%d\n",SNPpos[k],SNPpos[ TARGETsnp[k] ],qx,qr,LD[k],(float)x1,i);
		if( qr>0.1 && qx>0.1 ) 	{	
			meanr2[dcat][0]+= 1.0;
			meanr2[dcat][1]+= x1;
			if( dcat==5 && TARGETsnp[ TARGETsnp[k] ] == -1) {
				meanr2[6][0]+= 1.0;
				meanr2[6][1]+= x1; }
			if( dcat==5 && TARGETsnp[ TARGETsnp[k] ] == -2) {
				meanr2[7][0]+= 1.0;
				meanr2[7][1]+= x1; }
			if( dcat==5 && TARGETsnp[ TARGETsnp[k] ] == -3) {
				meanr2[8][0]+= 1.0;
				meanr2[8][1]+= x1; }

		}

        } // end    if  TARGETsnp[k]
   } //snps

for(k=0;k<9;k++) {
	meanr2[k][1] = meanr2[k][1]/meanr2[k][0];
	printf("LD cat %ld n %lf mean r2 %lf\n",k,meanr2[k][0],meanr2[k][1]); }

}






void FitnessVar(int rp, FILE * fd, pop_gen_t male_pop, pop_gen_t fem_pop) // prob of selected locus proportional to chrom length
{
	int i,j;
	long k;
    double xstats[10];    
    FITNESSM();
    FITNESSF();
    for(j=0;j<10;j++) xstats[j]=0.0;

    for(j=0;j<nMales;j++) {
	xstats[0]+=fitnessValueM[j]/(float)nMales;
	xstats[1]+=(fitnessValueM[j]*fitnessValueM[j])/(float)nMales;}

    for(j=0;j<nFemales;j++){
	xstats[2]+=fitnessValueF[j]/(float)nFemales;
	xstats[3]+=(fitnessValueF[j]*fitnessValueF[j])/(float)nFemales;}

   for(k=0;k<nSNPs;k++){
	if(SNPtype[k]==1 || SNPtype[k]==-1) {
		//fprintf(fd,"%d %ld %f ",rp,k,QQ[k]);
		// sum over males
		for(j=0;j<10;j++) xstats[j]=0.0;
		for (i = 0; i < nMales; ++i) {
		    if(k<FirstSNP[4]) j = bit_as_char(male_pop[i][0], k) + bit_as_char(male_pop[i][1], k);  // alleles
		    else j = bit_as_char(male_pop[i][1], k); // X-chrom for males
		    xstats[j]+=1;
		    xstats[3+j]+=fitnessValueM[i];	
		}
		for(j=0;j<3;j++) { 
			if(xstats[j]>0) xstats[3+j]=xstats[3+j]/xstats[j];
			xstats[8]+= (xstats[j]*xstats[3+j]/(float)nMales);
			xstats[9]+= (xstats[j]*xstats[3+j]*xstats[3+j]/(float)nMales); 
		}
		
		// females
		for(j=0;j<10;j++) xstats[j]=0.0;
		for (i = 0; i < nFemales; ++i) {
		    j = ( bit_as_char(fem_pop[i][0], k) + bit_as_char(fem_pop[i][1], k) );
		    xstats[j]+=1;
		    xstats[3+j]+=fitnessValueF[i];	
		}
		for(j=0;j<3;j++) { 
			if(xstats[j]>0) xstats[3+j]=xstats[3+j]/xstats[j];
			xstats[8]+= (xstats[j]*xstats[3+j]/(float)nFemales);
			xstats[9]+= (xstats[j]*xstats[3+j]*xstats[3+j]/(float)nFemales);
		}
	}
   }
}


void doSimRep(int rp, FILE * fd, FILE * fe) {
	int j;
	long  k, gen;
	double rx,ry,x1,x2,mA,mD;

    Founding_Pop();    //Founding population

	sum_diploid(0, FirstSNP[4], nMales, Qm, male_pop);
	sum_haploid(FirstSNP[4], nSNPs, nMales, Qm, male_pop);
	sum_diploid(0, nSNPs, nFemales, Qf, fem_pop);
    	for(k=0; k < nSNPs; k++) {        
        	rx = (Qm[k]+Qf[k] )/2.0;
		if(rp==0) QQrp0[k][0]=rx;
		if(rx<=0) QQsrt[k][rp]=0.0;
		else if(rx>=1) QQsrt[k][rp]=1.0;
		else {
			QQsrt[k][rp]=0.0; // allele frequency within sampled starting bulk
			for(j=0;j<(int)AllelesPerBulk1; j++) if(rand_uniform()<rx) QQsrt[k][rp]+=(1.0/(float)AllelesPerBulk1);
		}
	}

    printf("Rep %d SNPs %d\n",rp,nSNPs);
    //FitnessVar(rp,fd, male_pop,fem_pop);
    for(gen=0;gen<nGen;gen++) {
        simulateGen(gen);
	if(rp==0) {	// detailed allele frequency recording for rep0
		sum_diploid(0, FirstSNP[4], nMales, Qm, male_pop); // calc Qm
		sum_haploid(FirstSNP[4], nSNPs, nMales, Qm, male_pop); // calc Qm
		sum_diploid(0, nSNPs, nFemales, Qf, fem_pop); // calc Qf
	    	for(k=0; k < nSNPs; k++) {        
			rx = (Qm[k]+Qf[k] )/2.0;
			if(rx<=0.0) QQrp0[k][gen+1]=0.0;
			else if(rx>=1.0) QQrp0[k][gen+1]=1.0;
			else QQrp0[k][gen+1]= rx;
		}
	}
    }
	for(k=0; k < nSNPs; k++) QQrp0[k][19] =QQrp0[k][nGen] - QQrp0[k][0]; // total change in frequency
    //FitnessVar(rp,fd, male_pop,fem_pop);

	sum_diploid(0, FirstSNP[4], nMales, Qm, male_pop); // calc Qm
	sum_haploid(FirstSNP[4], nSNPs, nMales, Qm, male_pop); // calc Qm
	sum_diploid(0, nSNPs, nFemales, Qf, fem_pop); // calc Qf
    	for(k=0; k < nSNPs; k++) {        
        	rx = (Qm[k]+Qf[k] )/2.0;
		if(rx<=0.0) QQend[k][rp]=0.0;
		else if(rx>=1.0) QQend[k][rp]=1.0;
		else {
			QQend[k][rp]=0.0; // allele frequency within sampled end bulk
			for(j=0;j<(int)AllelesPerBulk2; j++) if(rand_uniform()<rx) QQend[k][rp]+=(1.0/(float)AllelesPerBulk2);
		}
	}

	for(k=0; k < nSNPs; k++) {        
		if(rp==0){
			mA=SNPm[k][0];// read depth ancestral
			mD=SNPm[k][3];// read depth derived
		} else if(rp==1) {
			mA=SNPm[k][1];// read depth ancestral
			mD=SNPm[k][4];// read depth derived
		} else if(rp==2) {
			mA=SNPm[k][2];// read depth ancestral
			mD=SNPm[k][5];// read depth derived
		}
// sample of finite reads::
        	SNPrefA[k][rp]=0;
		for(j=0;j<mA;j++) if(rand_uniform()<QQsrt[k][rp]) SNPrefA[k][rp]++;
		SNPaltA[k][rp] = mA-SNPrefA[k][rp];
        	SNPrefD[k][rp]=0;
		for(j=0;j<mD;j++) if(rand_uniform()<QQend[k][rp]) SNPrefD[k][rp]++;
		SNPaltD[k][rp] = mD-SNPrefD[k][rp];
	}
 
    
} // void doSimRep

void do_sim_for_indiv(ind_gen_t ng, int is_male, int rsx);
void bulletproof_copy_haplotype(const core_haplotype * par_hap, core_haplotype * child_hap, unsigned int start, unsigned int end);
void copy_haplotype(const core_haplotype * par_hap, core_haplotype * child_hap, unsigned int start, unsigned int end);
int rand_bool();
void copy_haplotype(const core_haplotype * par_hap, core_haplotype * child_hap, unsigned int start, unsigned int end);

void bulletproof_copy_haplotype(const core_haplotype * par_hap, core_haplotype * child_hap, unsigned int start, unsigned int end) {
    int k;
    for (k = start; k < end; ++k) {
        copy_bit(par_hap, k, child_hap);
    }
}

#define FANCY_COPY 1
void copy_haplotype(const core_haplotype * par_hap, core_haplotype * child_hap, unsigned int start, unsigned int end) {
# if defined(FANCY_COPY) && FANCY_COPY
    if (end - start < 16) {
        bulletproof_copy_haplotype(par_hap, child_hap, start, end);
        return;
    }
    unsigned int k, whole_byte_ind, fake_end, end_whole_byte_ind;
    // copy bits of the first non-whole byte
    if (start % 8) {
        whole_byte_ind = 1 + (start/8);
        fake_end = 8 * whole_byte_ind;
        assert (fake_end <= end);
        bulletproof_copy_haplotype(par_hap, child_hap, start, fake_end);
    } else {
        whole_byte_ind = start/8;
    }
    end_whole_byte_ind = 1 + (end/8);
    for (k = whole_byte_ind; k < end_whole_byte_ind; ++k) {
        child_hap[k] = par_hap[k]; // 8 snps at a time
    }
    // copy bits of the last non-whole byte
    if (end % 8) {
        whole_byte_ind = 8 * (end/8);
        bulletproof_copy_haplotype(par_hap, child_hap, whole_byte_ind, end);
    }
#else
    bulletproof_copy_haplotype(par_hap, child_hap, start, end);
#endif
}

int rand_bool() {
    return (int)( 2.0*rand_uniform() );  // start haplotype (0 or 1)
}



unsigned int select_indiv(unsigned int n, const double * cumulative_array) {
    const double rl = rand_uniform();   // draw uniform random value 
    unsigned int i = 0;
    for (i = 0; i < n; ++i) {
        if (cumulative_array[i] >= rl) {
            return i;
        }
    }
    return 0;
}

inline unsigned int selectFitM() {
    return select_indiv(nMales, cumulativeValueM);
}

inline unsigned int selectFitF() {
    return select_indiv(nFemales, cumulativeValueF);
}

void do_sim_for_indiv(ind_gen_t ng, int is_male, int rsx) {
    int cc, ccM,ccF, cc0,j; 
    long break_point,start_p,end_p,lowpos,highpos;
	float rx;
    ind_gen_t dad_gen, mom_gen;
    ccM = selectFitM();    //Selected dad
    dad_gen = male_pop[ccM];
    ccF = selectFitF();    //Selected female
    mom_gen = fem_pop[ccF];
	if(rsx==1) { // record successes
		RealizedW[ccM]++;
		RealizedW[1000+ccF]++;}


    cc = rand_bool();
    copy_haplotype(dad_gen[cc], ng[0], 0, FirstSNP[2]);
    cc = rand_bool();
    copy_haplotype(dad_gen[cc], ng[0], FirstSNP[2], FirstSNP[4]);
    if (is_male == 0) {
        copy_haplotype(dad_gen[1], ng[0], FirstSNP[4], nSNPs);
    }
	// vx change here


    cc0 = rand_bool(); // start allele chrom 2
	start_p=0;
	end_p=FirstSNP[1];
	if( rand_uniform() < chromRtotal[0] ) {  // crossover on this allele
		rx=rand_uniform();
		j=0;
		lowpos=start_p;
		while(j<100){	
			if(rx<Rvals[0][j]) {
				highpos=Rpoints[0][j];
				j=100;  }
			else {	j++;
				lowpos=Rpoints[0][j]; }
		}
		break_point = lowpos + (int)( (highpos - lowpos) *rand_uniform() );
		
		copy_haplotype(mom_gen[cc0], ng[1], start_p, break_point);
		cc0=1-cc0;
		copy_haplotype(mom_gen[cc0], ng[1], break_point, end_p);
	}
	else {	
		copy_haplotype(mom_gen[cc0], ng[1], start_p, end_p); }
	start_p=FirstSNP[1];
	end_p= FirstSNP[2];
	if( rand_uniform() < chromRtotal[1] ) {  // crossover on this allele
		rx=rand_uniform();
		j=0;
		lowpos=start_p;
		while(j<100){	
			if(rx<Rvals[1][j]) {
				highpos=Rpoints[1][j];
				j=100;  }
			else {	j++;
				lowpos=Rpoints[1][j]; }
		}
		break_point = lowpos + (int)( (highpos - lowpos) *rand_uniform() );		
		
		copy_haplotype(mom_gen[cc0], ng[1], start_p, break_point);
		cc0=1-cc0;
		copy_haplotype(mom_gen[cc0], ng[1], break_point, end_p);
	}
	else 	copy_haplotype(mom_gen[cc0], ng[1], start_p, end_p);

    cc0 = rand_bool(); // start allele chrom 3
	start_p=FirstSNP[2];
	end_p= FirstSNP[3];
	if( rand_uniform() < chromRtotal[2] ) {  // crossover on this allele
		rx=rand_uniform();
		j=0;
		lowpos=start_p;
		while(j<100){	
			if(rx<Rvals[2][j]) {
				highpos=Rpoints[2][j];
				j=100;  }
			else {	j++;
				lowpos=Rpoints[2][j]; }
		}
		break_point = lowpos + (int)( (highpos - lowpos) *rand_uniform() );		

		copy_haplotype(mom_gen[cc0], ng[1], start_p, break_point);
		cc0=1-cc0;
		copy_haplotype(mom_gen[cc0], ng[1], break_point, end_p);
	}
	else 	copy_haplotype(mom_gen[cc0], ng[1], start_p, end_p);
	start_p=FirstSNP[3];
	end_p= FirstSNP[4];
	if( rand_uniform() < chromRtotal[3] ) {  // crossover on this allele
		rx=rand_uniform();
		j=0;
		lowpos=start_p;
		while(j<100){	
			if(rx<Rvals[3][j]) {
				highpos=Rpoints[3][j];
				j=100;  }
			else {	j++;
				lowpos=Rpoints[3][j]; }
		}
		break_point = lowpos + (int)( (highpos - lowpos) *rand_uniform() );		

		copy_haplotype(mom_gen[cc0], ng[1], start_p, break_point);
		cc0=1-cc0;
		copy_haplotype(mom_gen[cc0], ng[1], break_point, end_p);
	}
	else 	copy_haplotype(mom_gen[cc0], ng[1], start_p, end_p);

    cc0 = rand_bool(); // start allele X
	start_p=FirstSNP[4];
	end_p= nSNPs;
	if( rand_uniform() < chromRtotal[4] ) {  // crossover on this allele
		rx=rand_uniform();
		j=0;
		lowpos=start_p;
		while(j<100){	
			if(rx<Rvals[4][j]) {
				highpos=Rpoints[4][j];
				j=100;  }
			else {	j++;
				lowpos=Rpoints[4][j]; }
		}
		break_point = lowpos + (int)( (highpos - lowpos) *rand_uniform() );		

		copy_haplotype(mom_gen[cc0], ng[1], start_p, break_point);
		cc0=1-cc0;
		copy_haplotype(mom_gen[cc0], ng[1], break_point, end_p);
	}
	else 	copy_haplotype(mom_gen[cc0], ng[1], start_p, end_p);

        
}

void advance_for_next_gen(unsigned int n, const pop_gen_t src, pop_gen_t dest);

void advance_for_next_gen(unsigned int n, const pop_gen_t src, pop_gen_t dest) {
    int i;
    ind_gen_t tmp;
    for (i = 0 ; i < n; ++i) {
        tmp = src[i];
        src[i] = dest[i];
        dest[i] = tmp;
    }
}

void calc_fitness(unsigned int n, double * cumulative_array, double * fitness_array, pop_gen_t pop, unsigned int diploid_end) {
    unsigned int i, k,js=0, curRank = 0;
    double prevCul=0.0,zu[n],cutoff=0.0,meanz=0.0;
	int rank[n];
if (SelectionType==0) // updates here for truncation selection
{
    for (i = 0; i < n; ++i) {
        zu[i] = 0.0;
	js=0;
        for (k = 0; k < diploid_end; ++k) {
            if (SNPtype[k] == 1) {
                zu[i] += (double)(bit_as_char(pop[i][0], k) + bit_as_char(pop[i][1], k))*slocus[js];
		js++;
            }
	    else if (SNPtype[k] == -1) {
                zu[i] -= (double)(bit_as_char(pop[i][0], k) + bit_as_char(pop[i][1], k))*slocus[js];
		js++;
            }
        }
        for (; k < nSNPs; ++k) {
            if (SNPtype[k] == 1) {
                zu[i] += (double)(bit_as_char(pop[i][1], k))*slocus[js]*2.0; // X-chrom for males
		js++;
            }
	    else if (SNPtype[k] == -1) {
                zu[i] -= (double)(bit_as_char(pop[i][1], k))*slocus[js]*2.0; // X-chrom for males
		js++;
            }
        }
	zu[i]+=0.01*rand_uniform(); // insures differences
	meanz+=zu[i]/(double)n;
	//printf("trait %lf ",zu[i]);
    }

	
	for (i = 0; i < n; i++) {
		rank[i]=0;
		for(k=0;k<n;k++) if(zu[k]>zu[i]) rank[i]++; // low ranks are good
		if( rank[i]==( (int)((float)n*fraction_selected))-1) cutoff= zu[i];
	}
	
	js=0; // no selected
	for (i = 0; i < n; i++) {
		if (zu[i]>=cutoff) {	fitness_array[i] = 1.0;
					js+=1;}
		else fitness_array[i] = 0.0; }
	printf("selected %d meanZ %lf cutoff %lf\n",js,meanz,cutoff);

	for (i = 0; i < n; ++i) {
	cumulative_array[i] = prevCul + fitness_array[i];
	prevCul = cumulative_array[i];
	}
	for (i = 0; i < n; ++i) {
	cumulative_array[i] = cumulative_array[i]/prevCul;
	}
}
else if (SelectionType==1) 
{
    for (i = 0; i < n; ++i) {
        zu[i] = 1.0;
	js=0;
        for (k = 0; k < diploid_end; ++k) {
            if (SNPtype[k] == 1) {
                zu[i] *=(1.0+ (double)(bit_as_char(pop[i][0], k) + bit_as_char(pop[i][1], k))*slocus[js] );
		js++;
            }
	    else if (SNPtype[k] == -1) {
                zu[i] *=(1.0- (double)(bit_as_char(pop[i][0], k) + bit_as_char(pop[i][1], k))*slocus[js] );
		js++;
            }
        }
        for (; k < nSNPs; ++k) {
            if (SNPtype[k] == 1) {
                zu[i] *=(1.0+ (double)(bit_as_char(pop[i][1], k))*slocus[js]*2.0 ); // X-chrom for males
		js++;
            }
	    else if (SNPtype[k] == -1) {
                zu[i] *=(1.0- (double)(bit_as_char(pop[i][1], k))*slocus[js]*2.0 ); // X-chrom for males
		js++;
            }
        }
	meanz+=zu[i]/(double)n;
	//printf("trait %lf ",zu[i]);
    }

	for (i = 0; i < n; i++) fitness_array[i] = zu[i];

	printf("selected %d meanZ %lf\n",js,meanz);

    for (i = 0; i < n; ++i) {
        cumulative_array[i] = prevCul + fitness_array[i];
        prevCul = cumulative_array[i];
    }
    for (i = 0; i < n; ++i) {
        cumulative_array[i] = cumulative_array[i]/prevCul;
    }

}

} // end calc_fitness

inline void FITNESSM()
{    
    calc_fitness(nMales, cumulativeValueM, fitnessValueM, male_pop, FirstSNP[4]);
}

inline void FITNESSF()
{
    calc_fitness(nFemales, cumulativeValueF, fitnessValueF, fem_pop, nSNPs);
}

void simulateGen(long gen) {
    int i,kx;
    float mm=0.0,mv=0.0;
    FITNESSM();
    FITNESSF();

	if(SpitW==1 && gen==9) {	for(i=0;i<2000;i++) RealizedW[i]=0;
					kx=1;
	}
	else kx=0;
    for(i = 0 ; i < nMales; i++) {
        do_sim_for_indiv(n_male_pop[i], 1, kx);
    }
    for(i = 0 ; i < nFemales; i++) { 
        do_sim_for_indiv(n_fem_pop[i], 0, kx);
    }

	// printf("Males mean %f var %f\n",mm,mv-(mm*mm));


    advance_for_next_gen(nMales, n_male_pop, male_pop);
    advance_for_next_gen(nFemales, n_fem_pop, fem_pop);
}

int POISSON(double m) {
    double rx,pr,cprob,u1,u2,z1;
    float pi = 3.1415927;
    int j;
    int L=5000; //max
    if(m<200) {
        rx =rand_uniform();	// Draw random value to select winner
        pr = exp(-m);//prob of zero mutations
        cprob = pr;
        for(j=0;j<=L;j++) {
            if (rx < cprob) {
                break;
            }
            pr = pr*m/(double)(j+1);
            cprob += pr;
        }
        return j;
    } 
    //use normal approx
    u1=rand_uniform();
    u2=rand_uniform();
    z1 = sqrt(-2*log(u1))*cos(2*pi*u2);
    z1 = m + sqrt(m)*z1;
    return (int)(z1+0.5);
}




void Founding_Pop() {
    long i,j,k=0,k2=0,x;
    double z,prob1;
    int allele,lastallele;
    int AllelesList[nSNPs];
    
	for(i=0;i<nSNPs;i++) AllelesList[i]=0;

    for(i=0;i<nMales;i++) { // Individual males
        for(j=0;j<2;j++) {  // haplotype

	// target loci

            for(x=0;x<NoTargets;x++) {
                k=List_o_targets[x];
		if(TARGETsnp[k] >= 0) printf("Error 673 here\n");
                if(rand_uniform() < QQ[k])   AllelesList[k]=1;
                else            AllelesList[k]=0;
                set_bit(male_pop[i][j], k, AllelesList[k]); }

	// rest of loci given target genotypes
            for(k=0;k<nSNPs;k++) {   // snp
                   if (k<FirstSNP[4] || j==1) {

                	if(TARGETsnp[k] >= 0) // not a focal snp
			{	lastallele=AllelesList[ TARGETsnp[k] ];
				// LD[k] is between minor alleles
				if ( (QQ[k]>=0.5 && QQ[ TARGETsnp[k] ]>=0.5) || (QQ[k]<=0.5 && QQ[ TARGETsnp[k] ]<=0.5) ) {
					if (lastallele==1) prob1 = (QQ[k]*QQ[ TARGETsnp[k] ]+LD[k])/QQ[TARGETsnp[k]];
					else prob1 = (QQ[k]*(1-QQ[TARGETsnp[k]])-LD[k])/(1.0-QQ[TARGETsnp[k]]); }
				else {
					if (lastallele==1) prob1 = (QQ[k]*QQ[ TARGETsnp[k] ]-LD[k])/QQ[ TARGETsnp[k] ];
					else prob1 = (QQ[k]*(1-QQ[ TARGETsnp[k] ])+LD[k])/(1-QQ[TARGETsnp[k]]); }
		                if(rand_uniform() < prob1) AllelesList[k]=1;
		                else  AllelesList[k]=0;
		        	set_bit(male_pop[i][j], k, AllelesList[k]);
			}
		   }
            }
        }
    }

    for(i=0;i<nFemales;i++) { // Individual females
        for(j=0;j<2;j++) {   // haplotype
               
            for(x=0;x<NoTargets;x++) {
                k=List_o_targets[x];
		if(TARGETsnp[k] >= 0) printf("Error 673 here\n");
                if(rand_uniform() < QQ[k])   AllelesList[k]=1;
                else            AllelesList[k]=0;
                set_bit(fem_pop[i][j], k, AllelesList[k]); }


            for(k=0;k<nSNPs;k++) {

                	if(TARGETsnp[k] >= 0) // not a focal snp
			{	lastallele=AllelesList[ TARGETsnp[k] ];
				// LD[k] is between minor alleles
				if ( (QQ[k]>=0.5 && QQ[ TARGETsnp[k] ]>=0.5) || (QQ[k]<=0.5 && QQ[ TARGETsnp[k] ]<=0.5) ) {
					if (lastallele==1) prob1 = (QQ[k]*QQ[ TARGETsnp[k] ]+LD[k])/QQ[TARGETsnp[k]];
					else prob1 = (QQ[k]*(1-QQ[TARGETsnp[k]])-LD[k])/(1.0-QQ[TARGETsnp[k]]); }
				else {
					if (lastallele==1) prob1 = (QQ[k]*QQ[ TARGETsnp[k] ]-LD[k])/QQ[ TARGETsnp[k] ];
					else prob1 = (QQ[k]*(1-QQ[ TARGETsnp[k] ])+LD[k])/(1-QQ[TARGETsnp[k]]); }
		                if(rand_uniform() < prob1) AllelesList[k]=1;
		                else  AllelesList[k]=0;
		        	set_bit(fem_pop[i][j], k, AllelesList[k]);
			}
            }
        } //j
    } // i

}// end function




double generateGaussianNoise(double mu, double sigma,double u1,double u2) {
	static const double two_pi = 2.0*3.14159265358979323846;
	double z0;
	z0 = sqrt(-2.0 * log(u1)) * cos(two_pi * u2);
	/* z1 = sqrt(-2.0 * log(u1)) * sin(two_pi * u2); */
	return z0 * sigma + mu;
}

